/*
**
**      File : nrd.c 
**      (C)1996-97  Mike Flouris.
**
**      Description : Code implementing a prototype
**		device driver for the nrd (Network RAM-Disk)
**
*/

#include <sys/machine/devdriver.h>
#include <machine/iotypes.h>
#include <machine/pt.h>

#include <sys/uio.h>
#include <kern/thread.h>
#include <kern/task.h>
#include <mach/vm_param.h>
#include <vm/vm_page.h>
#include <vm/vm_object.h>
#include <vm/vm_kern.h>
#include <sys/buf.h>
#include <sys/disklabel.h>
#include <sys/ioctl.h>
#include <io/common/devio.h>
#include <sys/ioctl_compat.h>
#include <sys/machine/srvc.h>
#include <sys/machine/devgeteei.h>
#include <sys/machine/devgetinfo.h>

#include <ufs/fs_proto.h>

#include <sys/socket.h>                 /* Socket related definitions */
#include <sys/socketvar.h>              /* Socket related definitions */
#include <netinet/in.h>                 /* Sockaddr_in definition */
#include <net/route.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>
#include <netinet/ip_var.h>
#include <netinet/in_pcb.h>             /* Definition of sotoinpcb */
#include <netinet/tcp.h>
#include <netinet/tcpip.h>
#include <netinet/tcp_timer.h>
#include <netinet/tcp_var.h>    /* Definition of intotcpcb */
#include <sys/mbuf.h>

#include "nrd.h"		/* Config header file ... */

/*
** #########################
**	Global Variables
** ######################### */

/* How many times is it open ? */
int open_nrd = 0;

/* Is nrd initialized ? */
int nrd_is_initialized = NO;

/* Queue Flag (must be 0 here) */
int nrd_queue = 0;

/* Stop Ioctl Flag */
int stop_cnt  = 0;

/* Dummy event used to detect timeout */
event_t	e;                              

/* Proxy global variables ... */
struct  buf             nrd_bufq;                       /* Queue of pending requests */
struct  buf             *nrd_head = &nrd_bufq;          /* Head of the queue of pending requests */
static  boolean_t       proxy_is_sleeping = TRUE;       /* Is proxy thread asleep? */

/* Variables for speed tests */
#ifdef NRDSPEED
int before = 0, after = 0;
#endif
#ifdef NRD_INSTRUMENT
struct timeval	nrd_tv1, nrd_tv2;
long 		nrd_time_1, nrd_time_2;
#endif

/*
** #####################
**   Disk Variables ...
** ##################### */
struct devget   *devget;
device_info_t   *devinfo;
struct disklabel *lp;
DEVGEOMST *devgeom;
struct pt *pt;
struct pt_tbl *ptp;
struct pt_info *def_pt;
struct Nrd_Server_Info *nrdserv;

/*
** #####################
**    Block Table ...
** ##################### */
Nrd_Block_Table_Entry blocktable [ NRDHOSTS ];

/*
** #####################
**	Host Table ...
** ##################### */
Nrd_Host_Entry nrdhosts[ NRDHOSTS ];

/* Host configuration routines ... */
#include "nrd_config.c" 

/* Socket Calls ... */
#include "nrd_sockets.c"

int nrdopen(), nrdstrategy(), nrdpsize(), nrdclose(), nrdioctl(), nrdread(), nrdwrite();
void nrdshutdown(), nrdflush();

/* Adding the files containing the code of the interfaces */
#include "nrd_open.c"
#include "nrd_close.c"
#include "nrd_queue.c"
#include "nrd_read.c"
#include "nrd_write.c"
#include "nrd_strategy.c"
#include "nrd_size.c"
#include "nrd_ioctl.c"

/* Shutdown Code ... */
void nrdshutdown (parameter) 
caddr_t parameter; 
{
	int i,t;

	t = NRD_REQ_STOP_SERVER;
	for (i = 0; i < NRDHOSTS; i++)
		nrd_send(nrdhosts[ i ].sockptr, (char *) &t, sizeof(char), 0);
		nrd_is_initialized = 0;
		open_nrd = 0;
	for (i = 0; i < NRDHOSTS; i++)
		soclose( nrdhosts[i].sockptr );

	/* Free some memory ... */
	kfree(devget, sizeof ( struct devget ) );
	kfree(devinfo, sizeof ( device_info_t ) );
	kfree(lp, sizeof ( struct disklabel ) );
	kfree(devgeom, sizeof ( DEVGEOMST ) );
	kfree(pt, sizeof ( struct pt ) );
	kfree(ptp, sizeof ( struct pt_tbl ) );
	kfree(def_pt, sizeof ( struct pt_info ) );

	uprintf("Let's go down ...\n");

	return;
}

/* Flush Code */
void nrdflush (parameter)
caddr_t parameter;
{

	uprintf("Flushing ... \n");

	return;
}


/*
	This is the End ...
*/
