/*
**
**      File : nrd.h 
**      (C)19961998  Mike Flouris.
**
**      Description : Header file for the device driver
**                    interfaces of the NRD
**      Some things MUST be configured here ...
**
*/

/*
** ################################
**    How many server hosts ???
** ################################ */
/* Configurable ... */
#define NRDHOSTS 2

/*
** ######################################
**   Size of NRD per server : 24 MBytes
** ###################################### */
#define NRD_HOST_BYTES  ( 24 * 1024 * 1024 )

/*
** ######################################################
**    The host addresses for our subnet (critical) ...
**    (the net addresses MUST be configured)
** ###################################################### */
#define ELECTRA         0x05bfa8c0              /* 192.168.191.5 */
#define LUCIFER         0x06bfa8c0              /* 192.168.191.6 */
#define ARAMIS          0x07bfa8c0              /* 192.168.191.7 */
#define PORTHOS         0x08bfa8c0              /* 192.168.191.8 */
#define NULLHOST    0x0
/*
 * The addresses are calculated using HEX numbers with
 * the reverse order :
 * 0x05bfa8c0 == 0x 05(=5) bf(=191) a8(=168) c0(=192), so
 * 0x05bfa8c0 == 192.168.191.5 (reverse order)
 */

/*
** ###############################################
**  NRD hosts & hostnames (critical) ...
**  The order of the hosts is defined here ...
** ############################################### */
#define NRD_HOST_1  PORTHOS /* PORTHOS defined above to address */
#define NRD_HOSTNAME_1  "porthos"
#define NRD_HOST_2  LUCIFER /* LUCIFER defined above to address */
#define NRD_HOSTNAME_2  "lucifer"
/*
#define NRD_HOST_2  ELECTRA
#define NRD_HOSTNAME_2  "electra"
*/
#define NRD_HOST_3  NULLHOST
#define NRD_HOSTNAME_3  ""
#define NRD_HOST_4  NULLHOST
#define NRD_HOSTNAME_4  ""

/*
** ####################################
**  The port number I use ...
**  The server must use the same port.
** #################################### */
/*#define PORTNUM         0x4908          /* Port number 2121  = 0x0849 */
#define PORTNUM         0x5b04          /* Port number 1115  = 0x5b04 */

/*
** #######################################################
**  The client's disk partition (not used, leave it ...)
** ####################################################### */
#define DISK_MAJOR              8   /* Major number of backup disk partition */
#define DISK_MINOR              1024    /* Minor number of backup disk partition */





/* ******************************************************************
   ##################################################################
        Config ends here. DO NOT edit below ...
   ##################################################################
   ******************************************************************/





/*
** ###################################
** Programming & Debugging Defines ...
** ###################################
*/
#define YES 1
#define NO  0

/* Debugging info level 1 (mostly garbage, leave it untouched) */
#undef NRDDEBUG
/*#define NRDDEBUG*/
#undef NRDDEBUGI
/*#define NRDDEBUGI*/

/* Debugging info for sockets */
#undef NRDDEBUGS
/*#define NRDDEBUGS*/

/* Count the Clock cycles ... */
#undef NRDSPEED
/*#define NRDSPEED*/

/* Time some parts ... */
#undef NRD_INSTRUMENT

/*
** #####################
**     Typedefs ...
** ##################### */
typedef struct Header {
    unsigned type;
    unsigned blocknum;
    unsigned stblk;
} Header;

typedef struct Nrd_Host_Entry {
    char hostname[16];      /* Name of the host */
    int  hostaddr;          /* Address of the host */
    int  portnum;           /* Port of the server */

    int  num_blocks;        /* Number of blocks it stores */
    int  num_bytes;         /* Number of bytes it stores */

    struct  socket  *sockptr;       /* Socket pointer */
} Nrd_Host_Entry;

typedef struct Nrd_Server_Info {
    int server_no;
    Nrd_Host_Entry server_data;
} Nrd_Server_Info;

typedef struct Nrd_Block_Table_Entry {
    int hostnum;            /* Number of the host (as in the nrdhosts) ... */

    int start_block;        /* Start block number for host ... */
    int end_block;          /* End block number for host ... */
} Nrd_Block_Table_Entry;


/* Use queue for write requests ? */
#define USE_QUEUE

/* Which block distribution to use ? */
#define SEQUENTIAL_BLOCK_DISTRIBUTION
/*#define STRIPED_BLOCK_DISTRIBUTION*/

/* NRD Size defines ... */
#define NRD_HOST_BLOCKS ( NRD_HOST_BYTES / DEV_BSIZE )
#define NRD_SIZE_BYTES  ( NRD_HOST_BYTES * NRDHOSTS )
#define NRD_SIZE_BLOCKS ( NRD_SIZE_BYTES / DEV_BSIZE )

/*
** ################################
**  TCP/IP defines ...
** ################################ */
#undef  SET_TCP_BUFFER          /* Maximize TCP buffer ??? (default: no) */
#define NRD_SOCKBUFSIZE  256*1024       /* Max TCP buffer size ... */
#define NRD_TIMEOUT             1024    /* Timeout to connect sockets to servers ... */
#define TIMEOUT                 1       /* Polling interval for send/rcv */
#define COUNT_MAX               60000   /* Connection timeout in ms*/

/*
** ################################
     Type of operation requested
** ################################ */
#define NRD_REQ_BLOCK_WRITE 0x0 /* Block write request */
#define NRD_REQ_BLOCK_READ  0x1 /* Block read  request */
#define NRD_REQ_BYTE_WRITE  0x2 /* Byte  write request */
#define NRD_REQ_BYTE_READ   0x3 /* Byte  read  request */
#define NRD_REQ_SAVE        0x4 /* Save NRD to server's disk */
#define NRD_REQ_RESTORE     0x5 /* Restore NRD from server's disk */
#define NRD_REQ_STATS           0x6     /* Server VM Stats request ... */
#define NRD_REQ_STOP_SERVER     0x7 /* Shutdown server request ... */

/*
** ############################################
**  Error Number for Lost connection 
** ############################################ */
#define ETIMEO      128 /* Error value: Connection timeout */

/*
** ##################################
     NRD special ioctl commands ... 
** ################################## */
    /* This restores NRD from disk (each server) ... */
#define NRD_IOCTL_RESTORE   _IO('N', 0)
    /* This saves NRD to disk file (each server) ... */
#define NRD_IOCTL_SAVE      _IO('N', 1)
    /* This queries NRD servers for VM statistics ... */
#define NRD_IOCTL_STATS     _IO('N', 2)
    /* This starts the queue ... */
#define NRD_IOCTL_QUEUE     _IO('N', 3)
#define NRD_IOCTL_NQUEUE    _IO('N', 4)
    /* This stops the server and the queue ... */
#define NRD_IOCTL_STOP      _IO('N', 5)
#define NRD_IOCTL_SERVERS   _IOW('N', 6, Nrd_Server_Info)

/*
** ##########################
**   External Functions ... 
** ########################## */
extern int  setdisklabel();
extern int  cycle_count();
extern void nrd_nodelay();
extern int  open_nrd_sockets();
extern int  nrd_rcv();
extern int  nrd_send();


