/*
**      File : nrd_ioctl.c
**      (C)1996-97  Mike Flouris.
**
**      Description : ioctl calls for the nrd (Network RamDisk)
**
*/

/*
 * IMPORTANT NOTE : Be VERY CAUTIOUS with ioctl commands. The machine
 *           will hang without any reason (with an kernel memory read
 *			    access error), without printing any debugging output like
 *			    printfs/uprintfs. When something like this happens, it's
 *			    surely an ioctl error ...
*/

int nrdioctl(dev, cmd, data, flag)
dev_t dev;		/* major/minor nunber */
register int cmd;	/* Ioctl command */
caddr_t data;           /* User data buffer - already copied in */
int flag;		/* unused */
{

        int  i = 0;
	char t = 0;
	struct vm_statistics    srv_vmstats[ NRDHOSTS ];

#ifdef NRDDEBUGI
		uprintf("NRD IOCTL (%d - %d) ... \n", major(dev), minor(dev) );
#endif

#ifdef NRDSPEED
	after = cycle_count();
        uprintf("Entering ioctl : %d\n", after - before );

	before = cycle_count();
#endif

	switch (cmd) {

	case DEVGETINFO:
#ifdef NRDDEBUGI
		uprintf("NRD IOCTL, DEVGETINFO\n");
#endif
		devinfo = (device_info_t *) data;
		bzero((caddr_t)devinfo, sizeof(device_info_t));
		devinfo->version = VERSION_1;
		devinfo->v1.version = VERSION_1;
		devinfo->v1.category = DEV_DISK;
		devinfo->v1.bus = DEV_UNKBUS;
		bcopy(DEV_SCSI_GEN, devinfo->v1.interface, strlen( DEV_SCSI_GEN ));
      bcopy("Network Ramdisk", devinfo->v1.device, strlen("Network Ramdisk") );
      bcopy("nrd", devinfo->v1.dev_name, 4 );
		devinfo->v1.soft_count = 0;
		devinfo->v1.hard_count = 0;
		devinfo->v1.businfo.adpt_num = -1;
		devinfo->v1.businfo.nexus_num = -1;
		devinfo->v1.businfo.bus_num = -1;
		devinfo->v1.businfo.ctlr_num = -1;
		devinfo->v1.businfo.rctlr_num = -1;
		devinfo->v1.businfo.slave_num = 0;
		devinfo->v1.businfo.unit_num = -1;
		devinfo->v1.businfo.bus.scsi.bus_num = 0;
		devinfo->v1.devinfo.disk.status = DKDEV_READY;
		devinfo->v1.devinfo.disk.class = DKDEV_CLS_HARDDISK;
		devinfo->v1.devinfo.disk.raid_level = 0;
		devinfo->v1.devinfo.disk.media_changes = 0;
		devinfo->v1.devinfo.disk.arch.scsi.flags = 0;
		devinfo->v1.devinfo.disk.capacity = NRD_SIZE_BLOCKS;
		devinfo->v1.devinfo.disk.blocksz = DEV_BSIZE;
		devinfo->v1.devinfo.disk.part_num = 0;
	break;

/*
	case DEVGETEEI:
		uprintf("NRD IOCTL, DEVGETEEI\n");
	break;
	*/

	case DEVIOCGET:
#ifdef NRDDEBUGI
		uprintf("NRD IOCTL DEVIOCGET cmd ... \n");
#endif
		devget = (struct devget *)data;
		bzero((caddr_t)devget,sizeof(struct devget));
		/*
		devgetinfo_2_devio(devinfo, devget);
		*/
		devget->category = DEV_DISK;
		devget->bus = DEV_UNKBUS;
		bcopy(DEV_SCSI_GEN, devget->interface, strlen(DEV_SCSI_GEN));
		bcopy("Network Ramdisk", devget->device, strlen("Network Ramdisk"));
		devget->adpt_num =  -1;
		devget->nexus_num =  -1;
		devget->bus_num =  -1;
		devget->ctlr_num = -1;
		devget->rctlr_num = -1;
		devget->slave_num = 0;
		bcopy("nrd", devget->dev_name, 4);
		devget->unit_num = -1;
		devget->soft_count = 0;
		devget->hard_count = 0;
		devget->stat = DEV_DONE;
		devget->category_stat = DEV_DISKPART;
	break;

	case DEVGETGEOM:
#ifdef NRDDEBUGI
		uprintf("NRD IOCTL , DEVIOCGET cmd ... \n");
#endif
		devgeom = (DEVGEOMST *)data;
		bzero((caddr_t)devgeom, sizeof(DEVGEOMST));
		devgeom->geom_info.dev_size = NRD_SIZE_BLOCKS;
		devgeom->geom_info.ntracks = (NRD_SIZE_BLOCKS / 2048);
		devgeom->geom_info.nsectors = 32;
		devgeom->geom_info.ncylinders = 64;
		/*devgeom->geom_info.attributes = DEVGEOM_DYNAMIC;*/
		devgeom->geom_info.attributes = 0x0 ;
		devgeom->geom_info.sector_size = DEV_BSIZE;
		devgeom->geom_info.min_trans = devgeom->geom_info.sector_size;
                devgeom->geom_info.max_trans = (128 * devgeom->geom_info.sector_size );
		devgeom->geom_info.prefer_trans = (64 * devgeom->geom_info.sector_size);
		break;

	case DEVROOT:
#ifdef NRDDEBUGI
                uprintf("*** You cannot mount this device as root !!!\n", cmd);
#endif
		return( EINVAL );
                break;

	case DIOCGETPT:
#ifdef NRDDEBUGI
		uprintf("NRD IOCTL , DIOCGETPT cmd ... \n");
#endif
		pt = (struct pt *)data;
		bzero((caddr_t)pt, sizeof(struct pt));
		pt->pt_magic = PT_MAGIC;
		pt->pt_valid = PT_VALID;
		pt->pt_part[0].pi_nblocks = NRD_SIZE_BLOCKS;
		pt->pt_part[0].pi_blkoff = 0x0;
		break;

	case DIOCDGTPT:
#ifdef NRDDEBUGI
                uprintf("NRD IOCTL , DIOCDGTPT cmd ... \n");
#endif
		pt->pt_magic = PT_MAGIC;
		pt->pt_valid = PT_VALID;
		pt->pt_part[0].pi_nblocks = NRD_SIZE_BLOCKS;
		pt->pt_part[0].pi_blkoff = 0x0;
		break;

	case DIOCSETPT:
#ifdef NRDDEBUGI
		uprintf("NRD IOCTL , DIOCSETPT cmd ... \n");
#endif
		pt->pt_magic = PT_MAGIC;
		pt->pt_valid = PT_VALID;
		pt->pt_part[0].pi_nblocks = NRD_SIZE_BLOCKS;
		pt->pt_part[0].pi_blkoff = 0x0;
		break;

	case DIOCGDINFO:
#ifdef NRDDEBUGI
		uprintf("NRD IOCTL (%d - %d), DIOCGDINFO cmd ... \n", major(dev), minor(dev) );
#endif
		if (lp->d_magic != DISKMAGIC) {
			uprintf("Bad magick number ...\n");
                        return(EINVAL);
		}
                else
                	*(struct disklabel *)data = *lp;

#ifdef NRDDEBUGI
		uprintf("After the copy ... \n");
#endif
		break;

	case DIOCSDINFO:
#ifdef NRDDEBUGI
		uprintf("NRD IOCTL , DIOCSDINFO cmd ... \n");
#endif
		if ((flag & FWRITE) == 0)
                        return(EBADF);
                else setdisklabel(lp, (struct disklabel *)data, 0);
#ifdef NRDDEBUGI
		uprintf("NRD IOCTL: I set the disk label ...\n");
#endif
		break;

	case DIOCWDINFO:
#ifdef NRDDEBUGI
		uprintf("NRD IOCTL , DIOCWDINFO cmd ... \n");
#endif
		if ((flag & FWRITE) == 0)
                        return(EBADF);
                else if (setdisklabel(lp, (struct disklabel *)data, 0) )
#ifdef NRDDEBUGI
			uprintf("NRD IOCTL: I set the disk label (setdisklabel = %d ) ...\n");
#else
			;
#endif
		break;

	case DIOCGPART:
#ifdef NRDDEBUGI
		uprintf("NRD IOCTL (%d - %d),  DIOCGPART cmd ... \n", major(dev), minor(dev) );
#endif
		if (lp->d_magic != DISKMAGIC)
                        return(EINVAL);
                else {
                        ((struct partinfo *)data)->disklab = lp;
                	((struct partinfo *)data)->part = &(*lp).d_partitions[ 0 ] ;

                }
		break;

	case DIOCRFORMAT:
#ifdef NRDDEBUGI
		uprintf("NRD IOCTL , DIOCRFORMAT cmd ... \n");
#endif
		break;

	case DIOCWFORMAT:
#ifdef NRDDEBUGI
		uprintf("NRD IOCTL , DIOCWFORMAT cmd ... \n");
#endif
		break;

	case DIOCSSTEP:
#ifdef NRDDEBUGI
		uprintf("NRD IOCTL , DIOCSSTEP cmd ... \n");
#endif
		break;

	case DIOCSRETRIES:
#ifdef NRDDEBUGI
		uprintf("NRD IOCTL , DIOCSRETRIES cmd ... \n");
#endif
		break;

	case DIOCWLABEL:
#ifdef NRDDEBUGI
		uprintf("NRD IOCTL , DIOCWLABEL cmd ... \n");
#endif
		if ((flag & FWRITE) == 0)
                        return(EBADF);
                else
                        *lp = *(struct disklabel *)data;
#ifdef NRDDEBUGI
		uprintf("NRD IOCTL: I set the disk label ...\n");
#endif
		break;

	case DIOCGDEFPT:
#ifdef NRDDEBUGI
		uprintf("NRD IOCTL , DIOCGDEFPT cmd ... \n");
#endif
		ptp = (struct pt_tbl *)data;

		for ( i = 0; i < MAXPARTITIONS; i++) {
                	ptp->d_partitions[i].p_size 	= (*lp).d_partitions[i].p_size;
			ptp->d_partitions[i].p_offset	= (*lp).d_partitions[i].p_offset;
			ptp->d_partitions[i].p_fsize	= (*lp).d_partitions[i].p_fsize;
			ptp->d_partitions[i].p_fstype	= (*lp).d_partitions[i].p_fstype;
			ptp->d_partitions[i].p_frag	= (*lp).d_partitions[i].p_frag;
			ptp->d_partitions[i].p_cpg	= (*lp).d_partitions[i].p_cpg;
		}
		break;

	case DIOCGCURPT:
#ifdef NRDDEBUGI
		uprintf("NRD IOCTL , DIOCGCURPT cmd ... \n");
#endif
		*(struct pt_tbl *) data = *(struct pt_tbl *) lp->d_partitions; 
	break;

			/* Make all servers restore from the file
			   they kept on disk ( IF they did !!!) .
			   WARNING : All data currently on memory will
				     be lost !!! */
	case NRD_IOCTL_RESTORE:
#ifdef NRDDEBUGI
		uprintf("NRD IOCTL , NRD_IOCTL_RESTORE cmd ... \n");
#endif
		t = NRD_REQ_RESTORE;
		for (i = 0; i < NRDHOSTS; i++) { 
			nrd_send(nrdhosts[ i ].sockptr, (char *) &t, sizeof(char), 0);
		}
	break;
			/* Make all servers save the data to
			   a file on disk ...
			   WARNING : All data previously saved on disk
				     will be lost !!! */
	case NRD_IOCTL_SAVE:
#ifdef NRDDEBUGI
		uprintf("NRD IOCTL , NRD_IOCTL_SAVE cmd ... \n");
#endif
		t = NRD_REQ_SAVE;
		for (i = 0; i < NRDHOSTS; i++) { 
			nrd_send(nrdhosts[ i ].sockptr, (char *) &t, sizeof(char), 0);
		}
	break;

	case NRD_IOCTL_STATS:
#ifdef NRDDEBUGI
		uprintf("NRD IOCTL , NRD_IOCTL_STATS cmd ... \n");
#endif
		t = NRD_REQ_STATS;
		for (i = 0; i < NRDHOSTS; i++) {
			nrd_send(nrdhosts[ i ].sockptr, (char *) &t, sizeof(char), 0);
			nrd_rcv(nrdhosts[ i ].sockptr, (char *)&(srv_vmstats[i]),
					13 * sizeof(long), 0);
		}
		for (i = 0; i < NRDHOSTS; i++) {
			uprintf("Server Name          : %s\n", nrdhosts[ i ].hostname);
			uprintf("Server number        : %d\n", i);
			uprintf("Page size            : %d bytes\n", 
					srv_vmstats[i].pagesize);
			uprintf("Number of pages : %d\n", srv_vmstats[i].active_count 
				+ srv_vmstats[i].inactive_count);
			uprintf("Number of free pages : %d\n", srv_vmstats[i].free_count);
			uprintf("Number of pagefaults : %d\n", srv_vmstats[i].faults);
			uprintf("======================================\n");
		}
	break;

        case NRD_IOCTL_QUEUE:
#ifdef NRDDEBUGI
		uprintf("NRD IOCTL , NRD_IOCTL_QUEUE cmd ... \n");
#endif
                if ( ! nrd_queue ) {
                        uprintf("Using the queue ... \n");
                        nrd_queue = 1;
                } else {
                        uprintf("I'm already using the queue ...\n");
                        return (ESUCCESS);
                }
                
                if ( nrd_queue ) {
                        uprintf("Starting Queue Handler ... \n");
                        nrd_queue_handler();        /* Called by user thread 1 */
                }
        break;

        case NRD_IOCTL_NQUEUE:
#ifdef NRDDEBUGI
		uprintf("NRD IOCTL , NRD_IOCTL_NQUEUE cmd ... \n");
#endif
		if ( nrd_queue ) {
                        uprintf("Stopping the queue ... \n");
			nrd_queue = 0;
		}
        break;

	case NRD_IOCTL_STOP:
#ifdef NRDDEBUGI
		uprintf("NRD IOCTL , NRD_IOCTL_STOP cmd ... \n");
#endif
		if ( stop_cnt == 0 ) {
			uprintf("Destroying the NRD ? Are you sure ?\n");
			uprintf("If you are, call this ioctl() again ...\n");
			stop_cnt++;
		}
		else if ( stop_cnt > 0 ) {
			uprintf("Okay, I'm destroying the NRD ...\n");
			if ( nrd_is_initialized ) {
				t = NRD_REQ_STOP_SERVER;
				for (i = 0; i < NRDHOSTS; i++)
					nrd_send(nrdhosts[ i ].sockptr, (char *) &t, sizeof(char), 0);
				nrd_is_initialized = 0;
				open_nrd = 0;

				/* Close the sockets */
				for (i = 0; i < NRDHOSTS; i++)
					soclose( nrdhosts[i].sockptr );

			        /* Free some memory ... */
			        kfree(devget, sizeof ( struct devget ) );
			        kfree(devinfo, sizeof ( device_info_t ) );
			        kfree(lp, sizeof ( struct disklabel ) );
			        kfree(devgeom, sizeof ( DEVGEOMST ) );
			        kfree(pt, sizeof ( struct pt ) );
			        kfree(ptp, sizeof ( struct pt_tbl ) );
			        kfree(def_pt, sizeof ( struct pt_info ) );
			} else 
				uprintf("Cannot destroy NRD. Not Initialized yet ...\n");
			stop_cnt = 0;
			}
	break;

	case NRD_IOCTL_SERVERS:
#ifdef NRDDEBUGI
                uprintf("NRD IOCTL , NRD_IOCTL_SERVERS cmd ... \n");
#endif
		if ( ! nrd_is_initialized ) {
			nrdserv = (Nrd_Server_Info *)data;
			bcopy( (char *) &(nrdserv->server_data.hostname[0]),
				 (char *) &(nrdhosts[ nrdserv->server_no ].hostname[0]), 8);
			nrdhosts[ nrdserv->server_no ].hostaddr = nrdserv->server_data.hostaddr;
		} else
			uprintf("NRD Busy. Cannot change server ...\n");
	break;

	/*
	case DEVCONSOLE:
		uprintf("NRD IOCTL, DEVCONSOLE cmd\n");
	break;

	case SRVC_REQUEST:
		uprintf("NRD IOCTL, SRVC_REQUESTcmd\n");
	break;
	*/

	default:
#ifdef NRDDEBUGI
                uprintf("NRD IOCTL , cmd = 0x%x ... \n", cmd);
#endif
      uprintf("NRD IOCTL , cmd = 0x%x ... \n", cmd);
		break;
	}

#ifdef NRDDEBUGI
        uprintf("Returning from NRD IOCTL ...\n");
#endif

#ifdef NRDSPEED
	after = cycle_count();

	uprintf("Exiting ioctl : %d\n", after - before );
        before = cycle_count();
#endif

	return (ESUCCESS);
}

