/*
**
**      File : nrd_open.c
**      (C)1996-97  Mike Flouris.
**
**      Description : open device driver interface
**         for the nrd (Network RAM-Disk)
**
*/

#include <sys/fcntl.h>
#include <kern/kalloc.h>

	/* 
	** dev : has the major - minor numbers ... 
	** flag : flags to determine if you wanna read or write ...
	** format : used only for device drivers which are both 
	**	    block and character ...
	*/
	
int nrdopen(dev, flag, format)
dev_t dev;
int flag;
int format;
{

	int i = 0;

#ifdef NRDSPEED
	after = cycle_count();
        uprintf("Entering open : %d\n", after - before );

        before = cycle_count();
#endif

#ifdef NRDDEBUG
        uprintf("Entering nrdopen ...\n");
      	if (flag & FREAD) 
		uprintf("NRD OPEN major %d minor %d for reading\n", major(dev), minor(dev));
	else if (flag & FWRITE) 
		uprintf("NRD OPEN major %d minor %d for writing\n", major(dev), minor(dev));
	else if (flag & FAPPEND)
		uprintf("NRD OPEN major %d minor %d to append\n", major(dev), minor(dev));
	else
		uprintf("NRD OPEN major %d minor %d(flag = %d)\n", major(dev), minor(dev), flag);
#endif

	/***********************************
	** Initialize the Network RamDisk
	************************************/
	if ( ! nrd_is_initialized ) {
#ifdef NRDDEBUG
		uprintf("Initialising NRD .. \n");
#endif
		devget = ( struct devget * ) kalloc ( sizeof ( struct devget ) );
		devinfo = ( device_info_t * ) kalloc ( sizeof ( device_info_t ) );
		lp = (struct disklabel * ) kalloc ( sizeof ( struct disklabel ) );
		devgeom = (DEVGEOMST * ) kalloc ( sizeof( DEVGEOMST ) );
		pt = ( struct pt * ) kalloc ( sizeof( struct pt ) );
		ptp = ( struct pt_tbl * ) kalloc ( sizeof( struct pt_tbl ) );
		def_pt = ( struct pt_info * ) kalloc ( sizeof( struct pt_info ) );

					/* Setting the server attributes ... */
		if ( ! host_config() )
					/* Open the sockets to the servers ... */
			if ( ! open_nrd_sockets( NRDHOSTS ) )
				nrd_is_initialized = YES;
			else {
				uprintf("*** open_nrd_sockets failed ...\n");
				return ( EIO );
			}
		else {
			uprintf("*** host_config failed ...\n");
			return ( EIO );
		}

		/* Registering shutdown & flush interfaces ... */
		drvr_register_shutdown(nrdshutdown, (caddr_t) NULL, DRVR_REGISTER);
		drvr_register_flush(nrdflush, (caddr_t) NULL, DRVR_REGISTER);

#ifdef (NRDDEBUG || NRDDEBUGS)
		uprintf("Initialized NRD Sockets ...\n");
#endif

	} /**** if ( ! nrd_initialized ) */


	/*
	** Open the device ...
	*/
	if ( open_nrd >= 0 ) 
		open_nrd++ ;
	else {
		uprintf("nrd_open: open_nrd < 0 ...\n");
		return (ENXIO);
	}

#ifdef	WRITE_TO_LOCAL_DISK
	if(cdisk_open(makedev(DISK_MAJOR, DISK_MINOR), 0, 0) != ESUCCESS) {
                uprintf("Cdisk_open failed\n");
		return (EIO);
	}
#endif


#ifdef NRDDEBUG
                uprintf("Returning from nrdopen ...\n");
#endif

#ifdef NRDSPEED
        after = cycle_count();

	uprintf("Exiting open : %d\n", after - before );
        before = cycle_count();
#endif

        return (ESUCCESS);
}

