/*
**      File : nrd_queue.c
**      (C)1996-97  Mike Flouris.
**
**      Description : queue handling calls for the nrd (Network RamDisk)
**
*/


/* ###########################################################################
	  Function:		nrd_queue_handler
	  Arguments:	None
	  Description:	
	  Service loop executed by a dedicated thread. It dequeues the
	next request, it contacts the block server and in case of write,
	sends data and calls biodone(). In case of read, enqueues request
	for the listener thread.
   ########################################################################### */

void	nrd_queue_handler(void)
{
	int	i= 0, j = 0; 
	int server_no = 0;
	Header req;
	struct	buf	*bp;
	char ack = 0;
	

	nrd_head->b_actf = nrd_head->b_actl = nrd_head;

	while( nrd_queue )
	{
#ifdef	NRDDEBUG
		uprintf("Write Proxy is ON : waiting for data ...\n");	
#endif  /*  NRDDEBUG  */
		while(nrd_head->b_actf == nrd_head) 
		{
			proxy_is_sleeping = TRUE;
			sleep(nrd_head, PCATCH);
		}
		proxy_is_sleeping = FALSE;

		bp = nrd_head->b_actf;	/* Dequeue bp  */
		nrd_head->b_actf = bp->b_actf;
		bp->b_actf->b_actl = nrd_head;
		

	if ( bp->b_flags & B_READ ) {	/* Read type of request */

				/* Which server has the data ? */
#ifdef SEQUENTIAL_BLOCK_DISTRIBUTION
		server_no = ( bp->b_blkno / NRD_HOST_BLOCKS ) ;
#elif STRIPED_BLOCK_DISTRIBUTION
                server_no = find_server( bp->b_blkno );
#endif

		req.type =     (NRD_REQ_BLOCK_READ);	     /* Read request ... */
		req.blocknum = ( bp->b_bcount / DEV_BSIZE ); /* How many blocks ??? */
		req.stblk = ( bp->b_blkno % NRD_HOST_BLOCKS );/* Starting Block ??? */

#ifdef TRACE_DUMP
		/* Debugging ... */
		uprintf("read  %d  %d\n", req.blocknum, req.stblk);
#endif

#ifdef NRD_INSTRUMENT
		microtime(&nrd_tv2);	/* Time in usec ... */
		nrd_time_1 = (1000000 * (nrd_tv2.tv_sec - nrd_tv1.tv_sec)) +
				(nrd_tv2.tv_usec - nrd_tv1.tv_usec);
		microtime(&nrd_tv1);
#endif

		/* Check if the requested blocks are in more than one server ... */
		if ( ( (bp->b_blkno + req.blocknum) / NRD_HOST_BLOCKS ) != server_no ) {

			for (i = bp->b_blkno; (i / NRD_HOST_BLOCKS ) == server_no; i++);

					/* Handle the blocks from the first server */
			req.blocknum = i - bp->b_blkno - 1;
			if ( nrd_request( (Header *) &req, server_no ) )
				uprintf("Error sending request to server ...\n");
			nrd_rcv(nrdhosts[ server_no ].sockptr, bp->b_un.b_addr, (req.blocknum * DEV_BSIZE), 0);

					/* Handle the blocks from the second server */
			server_no = ( i / NRD_HOST_BLOCKS );
			req.blocknum = ((bp->b_bcount / DEV_BSIZE) - req.blocknum);
			req.stblk = ( i % NRD_HOST_BLOCKS );
			if ( nrd_request( (Header *) &req, server_no ) )
				uprintf("Error sending request to server ...\n");
			nrd_rcv(nrdhosts[ server_no ].sockptr, (char *) ((&bp->b_un.b_addr) + ((i - bp->b_blkno - 1) * DEV_BSIZE)), (req.blocknum * DEV_BSIZE), 0);


			/* Blocks exist only on one server ... */
		} else {
		
		/* Send the request to the server */
		if ( nrd_request( (Header *) &req, server_no ) ) 
			uprintf("Error sending request to server ...\n");

		/* Do the really hard stuff ... */
		nrd_rcv(nrdhosts[ server_no ].sockptr, bp->b_un.b_addr, bp->b_bcount, 0);
		}

#ifdef NRD_INSTRUMENT
		microtime(&nrd_tv2);
		nrd_time_2 = (1000000 * (nrd_tv2.tv_sec - nrd_tv1.tv_sec)) +
				(nrd_tv2.tv_usec - nrd_tv1.tv_usec);
		uprintf("Reqtype: READ(%d bytes), Systime: %d usec, Nettime: %d usec\n", bp->b_bcount, nrd_time_1, nrd_time_2);
#endif
		/* The read was successful ... */
                bp->b_resid = 0;
		bp->b_flags |= B_DONE;

		biodone( bp );

	} else {	/* Write type of request */

						/* Choose server(s) */
		server_no = ( bp->b_blkno / NRD_HOST_BLOCKS ) ;

		req.type =  ( NRD_REQ_BLOCK_WRITE);         /* Write request ... */
                req.blocknum = ( bp->b_bcount / DEV_BSIZE );/* How many blocks ??? */
                req.stblk = ( bp->b_blkno % NRD_HOST_BLOCKS ); /*Starting Block ???*/

#ifdef NRD_INSTRUMENT
		microtime(&nrd_tv2);	/* Time in usec ... */
		nrd_time_1 = (1000000 * (nrd_tv2.tv_sec - nrd_tv1.tv_sec)) +
				(nrd_tv2.tv_usec - nrd_tv1.tv_usec);
		microtime(&nrd_tv1);
#endif

		/* Check if the requested blocks are in more than one server ... */
		if ( ( (bp->b_blkno + req.blocknum) / NRD_HOST_BLOCKS ) != server_no ) {

			for (i = bp->b_blkno; (i / NRD_HOST_BLOCKS ) == server_no; i++);

					/* Handle the blocks from the first server */
			req.blocknum = i - bp->b_blkno - 1;
			if ( nrd_request( (Header *) &req, server_no ) )
				uprintf("Error sending request to server ...\n");
			nrd_send(nrdhosts[ server_no ].sockptr, bp->b_un.b_addr, (req.blocknum * DEV_BSIZE), 0);

					/* Handle the blocks from the second server */
			server_no = ( i / NRD_HOST_BLOCKS );
			req.blocknum = ((bp->b_bcount / DEV_BSIZE) - req.blocknum);
			req.stblk = ( i % NRD_HOST_BLOCKS );
			if ( nrd_request( (Header *) &req, server_no ) )
				uprintf("Error sending request to server ...\n");
			nrd_send(nrdhosts[ server_no ].sockptr, (char *) ((&bp->b_un.b_addr) + ((i - bp->b_blkno - 1) * DEV_BSIZE)), (req.blocknum * DEV_BSIZE), 0);

		} else {
		
		/* Send the request to the server */
                if ( nrd_request( (Header *) &req, server_no ) )
                        uprintf("Error sending request to server ...\n");

                /* Do the really hard stuff ... */
                nrd_send(nrdhosts[ server_no ].sockptr, bp->b_un.b_addr, bp->b_bcount, 0);

#ifdef SYNC_WRITES
                                        /* Receive the acknowledgement ... */
                nrd_rcv(nrdhosts[ server_no ].sockptr, (char *)&ack, sizeof(char), 0);

                                        /* Check the acknowledgement ... */
                while ( ack == 0 ) {
                                        /* Resend the data ... */
                nrd_send(nrdhosts[ server_no ].sockptr, bp->b_un.b_addr, bp->b_bcount, 0);
                                        /* Receive the acknowledgement ... */
                nrd_rcv(nrdhosts[ server_no ].sockptr, (char *)&ack, sizeof(char), 0);
                }
#endif          /* SYNC_WRITES */

#ifdef  WRITE_TO_LOCAL_DISK
                bp->b_dev = makedev(DISK_MAJOR, DISK_MINOR);
                if ( cdisk_strategy(bp) != ESUCCESS) {
                        uprintf("Cdisk_strategy write failed ...\n");
                        bp->b_flags |= B_ERROR;
                        bp->b_error = ENOSPC;
                        return (ENOSPC);
                }
#endif
		} /* one server ... */

#ifdef NRD_INSTRUMENT
		microtime(&nrd_tv2);
		nrd_time_2 = (1000000 * (nrd_tv2.tv_sec - nrd_tv1.tv_sec)) +
				(nrd_tv2.tv_usec - nrd_tv1.tv_usec);
		uprintf("Reqtype: WRITE(%d bytes), Systime: %d usec, Nettime: %d usec\n", bp->b_bcount, nrd_time_1, nrd_time_2);
#endif
		/* The read was successful ... */
                /* the write was successful ...*/
                bp->b_resid = 0;
		bp->b_flags |= B_DONE;

		biodone(bp);

		} /* Write request */

	} /* while ... */

	uprintf("Pager is now dead ...\n");
	return;
}


/* ###########################################################
  	Function:	nrd_request
	Arguments:	Header req, Server Number
	Description:	Function asking the server requests.
   ########################################################### */

int nrd_request( request , server_no)
	Header	*request;
	int	server_no;
{

	Header  req, req1;
	char ack = 0;
	int int_flag = 0;


#ifdef  NRDDEBUGS
                uprintf("Entering nrd_request ... \n");
#endif  /*  NRDDEBUGS  */

	req.type = request->type;
	req.blocknum = request->blocknum;
	req.stblk = request->stblk;

	/* Acknowledgment from the server */
#ifdef SERVER_ACK
					/* Send the request to the server */
	while ( ack == 0 ) {
		nrd_send(nrdhosts[server_no].sockptr, (char *)&req, (3 * sizeof(int)), 0);
		nrd_rcv(nrdhosts[server_no].sockptr, (char *)&req1, (3 * sizeof(int)), 0);
		if ( (req.type == req1.type) && ( req.blocknum == req1.blocknum) &&
			( req.stblk == req1.stblk) ) {
				ack = 1;
		}
		else {
			uprintf("Block Read : Wrong request ...\n");
			uprintf("ques : %d|%d|%d\n", req.type, req.blocknum, req.stblk);
			uprintf("ans  : %d|%d|%d\n", req1.type, req1.blocknum, req1.stblk);
			ack = 0;
		}

	nrd_send(nrdhosts[ server_no ].sockptr, (char *)&ack, sizeof(char), 0);
	}       /* while */


        /* No Acknowledgment from the server */
#else
				/* Send the request to the server */
	nrd_send(nrdhosts[server_no].sockptr, (char *)&req, (3 * sizeof(int)), 0);

#endif

#ifdef  NRDDEBUGS
                uprintf("Exiting nrd_request ... \n");
#endif  /*  NRDDEBUGS  */

	return( ESUCCESS );
}


