/*
**      File : nrd_sockets.c
**      (C)1996-97  Mike Flouris.
**
**      Description : socket calls for the nrd (Network RamDisk)
** 		
**	Original Code from George Dramitinos ( dramit@ics.forth.gr )
**
*/


/* ####################################################
	Function:     nrd_nodelay
	Arguments:    Socket pointer
	Description:  Sets NODELAY flag for socket.
   #################################################### */

void    nrd_nodelay(struct socket *so)
{
        struct  tcpcb   *tp;
        struct  inpcb   *inp;
        int                             s;
	int                             i = NRD_SOCKBUFSIZE;
        
#ifdef  NRDDEBUG
        uprintf("Entering nrd_nodelay ...\n");
#endif  /*  NRDDEBUG  */

        s = splnet();
        if((inp = sotoinpcb(so)) == 0 || (tp = intotcpcb(inp)) == 0 )
        {
                splx(s);
                uprintf("Warning: nrd_flush: sotoinpcb or intotcpcb failed\n");
                return;
        }
        tp->t_flags |=  TF_NODELAY;
        splx(s);

#ifdef  SET_TCP_BUFFER
	if(sbreserve(&so->so_snd, i) == 0 || sbreserve(&so->so_rcv, i) == 0)
                printf("nrd_nodelay: Warning Could not increase socket buffer size\n");
#endif

#ifdef  NRDDEBUG
        uprintf("Returning from nrd_nodelay ...\n");
#endif  /*  NRDDEBUG  */

}


/* ###########################################
	Function : open_nrd_sockets
	Argument : Number of sockets to open
   ########################################### */

int open_nrd_sockets( num_nrdhosts )
    int num_nrdhosts;
{

	int j = 0, i = 0, error = 0, time = 0;
	struct  sockaddr_in     psaddr; /* Stores server address */
	struct  mbuf            *nam;   /* Stores psaddr for connection */


#ifdef  NRDDEBUG
        uprintf("Entering open_nrd_sockets ...\n");
#endif  /*  NRDDEBUG  */

for(j = 0; j < num_nrdhosts; j++) {
					/* Setting the Socket characteristics ... */
        psaddr.sin_len = sizeof(struct sockaddr_in);
        psaddr.sin_family = AF_INET; 			/* TCP/IP connection */
        psaddr.sin_port = PORTNUM;			/* Server port 2121 = 0x0849 */
        psaddr.sin_addr.s_addr = nrdhosts[j].hostaddr;  /* Server host  */
        for(i = 0; i < 8; i++)  psaddr.sin_zero[i] = 0;

#ifdef  NRDDEBUGS
        uprintf("Yes, I'll try to open a socket ...\n");
#endif  /*  NRDDEBUGS  */

						/* Create the socket ... */
        error = socreate(AF_INET, &(nrdhosts[j].sockptr), SOCK_STREAM, 0);

#ifdef  NRDDEBUGS
        uprintf("socreate returned %d\n",error);
        uprintf("Well, push your luck and try to connect it too...\n");
#endif  /*  NRDDEBUGS  */

						/* Setting NO DELAY */
        if(error != 0)  return(-1);
        nrd_nodelay( nrdhosts[j].sockptr );
                				/* Simulate sockargs() */
        nam = m_get(M_WAIT, MT_SONAME);
        if(nam == NULL) {
                soclose( nrdhosts[j].sockptr );
                return(-1);
        }

						/* Copy the Socket characteristics to nam ...*/
        nam->m_len = sizeof(struct sockaddr_in);
        error = bcopy((caddr_t)&psaddr, mtod(nam, caddr_t), 0x10);

#ifdef  NRDDEBUGS
        uprintf("bcopy returned 0x%x\n", error);
#endif  /*  NRDDEBUGS  */

						/* Close the socket if there is an error */
        if(error != 0) {
		uprintf("so error is 0x%x\n", error);
                soclose( nrdhosts[j].sockptr );
                return(-1);
        }
                                                  /* Try to connect the socket ... */
        error = soconnect(nrdhosts[j].sockptr, nam);
#ifdef  NRDDEBUGS
        uprintf("soconnect returned 0x%x sock state is 0x%x\n",error,
                           nrdhosts[j].sockptr->so_state);
#endif  /*  NRDDEBUGS  */

						/* Close the socket if there is an error */
        if(error != 0) {
		uprintf("so error is 0x%x\n", error);
                soclose( nrdhosts[j].sockptr );
                return(-1);
        }

#ifdef  NRDDEBUGS
        uprintf("Waiting for socket to connect ...\n");
	uprintf("** error = 0x%x ... \n", nrdhosts[j].sockptr->so_error );
#endif  /*  NRDDEBUGS  */
						/* Wait until socket connects to host */
		  time=0;
        while ((nrdhosts[j].sockptr->so_state & SS_ISCONNECTING) &&
                   (nrdhosts[j].sockptr->so_error == 0) && (time < NRD_TIMEOUT)) {
					 time++;
                sosleep(nrdhosts[j].sockptr, 
			(caddr_t) (nrdhosts[j].sockptr->so_timeo), PZERO+1, 100);
		  }

		  if (( time == NRD_TIMEOUT ) || (nrdhosts[j].sockptr->so_error != 0))
              printf("NRD : Connecting Server %s timed out.\n", nrdhosts[j].hostname);

#ifdef  NRDDEBUGS
        uprintf("Now socket state is 0x%x\n", nrdhosts[j].sockptr->so_state);
        uprintf("so error is 0x%x\n", nrdhosts[j].sockptr->so_error);
#endif  /*  NRDDEBUGS  */

        m_freem(nam);

#ifdef  NRDDEBUGS
	if ( ! error )
        	uprintf("Yesss, its connected ... ( WOW !!! )\n");
	else printf("Not connected ...\n");
#endif  /*  NRDDEBUGS  */

        }

#ifdef  NRDDEBUG
        uprintf("Returning from open_nrd_sockets ...\n");
#endif  /*  NRDDEBUG  */

	if ( ! error )
		return (0);
	else	return (-1);

}


/* ##########################################################################
	Function:             nrd_rcv
	Arguments:    Socket pointer, data buffer and size requested
	Description: Rcv data from server. Simulates the read system call.
   ########################################################################## */


int      nrd_rcv(struct socket *so, char *buf, unsigned int size, 
			unsigned int ignore_timeo)
{
        struct uio auio;
        struct iovec aiov;
        long  error = 0;
	int   count = 0;

#ifdef  NRDDEBUGS
                uprintf("Going into nrd_rcv ... \n");
#endif  /*  NRDDEBUGS  */

        if(size == 0) {
			uprintf("nrd_rcv: Called for data size 0\n");
			return (-1);
			}

        aiov.iov_base = (caddr_t)buf;
        aiov.iov_len = size;
        auio.uio_iov = &aiov;
        auio.uio_iovcnt = 1;
        auio.uio_resid = size;
        auio.uio_rw = UIO_READ;
        auio.uio_segflg = UIO_SYSSPACE;


        while(auio.uio_resid != 0) {    /* Keep reading... */
                aiov.iov_base += aiov.iov_len - auio.uio_resid ;
                aiov.iov_len = auio.uio_resid;

		if(!ignore_timeo)
                {
                        while(count ++ < COUNT_MAX)
                        {
                                if(so->so_rcv.sb_cc < auio.uio_resid)
                                {
#ifdef  NRDDEBUGS
				printf("nrd_rcv: Calling event_wait cc %d size %d\n", so->so_rcv.sb_cc, size);
#endif /* NRDDEBUGS */
				event_wait(&e, 1, TIMEOUT);
#ifdef  NRDDEBUGS
				printf("nrd_rcv: After event_wait cc %d size %d\n", so->so_rcv.sb_cc, size);
#endif /* NRDDEBUGS */
                                }
                                else    break;
                        }

                        if(count >= COUNT_MAX)
                        {
                                printf("nrd_rcv:Warning connection timeout\n");
                                return (-1);
                        }
                }

                error = soreceive(so, NULL, &auio, NULL, NULL, 0);

                if (error) uprintf("rp_rcv: Error %d , Lost connection\n", error);

        } /* while ... */

#ifdef  NRDDEBUGS
                uprintf("Exiting nrd_rcv ... \n");
#endif  /*  NRDDEBUGS  */

        return (error);
}


/* ###########################################################################
	Function:             nrd_send
	Arguments:    Socket pointer, data buffer and size, ignore timeout flag
	Description:  Send data to server. Simulates the write system call.
   ########################################################################### */

int  nrd_send(struct socket *so, char *buf, unsigned int size, 
		unsigned int ignore_timeo)
{
        struct uio auio;
        struct iovec aiov;
        long  error = 0;
	int     count = 0;

#ifdef  NRDDEBUGS
                uprintf("Going into nrd_send ... \n");
#endif  /*  NRDDEBUGS  */

        if(size == 0) {
		uprintf("nrd_send: Called for data size 0\n");
		return ( -1 );
	}

        aiov.iov_base = (caddr_t)buf;
        aiov.iov_len = size;
        auio.uio_iov = &aiov;
        auio.uio_iovcnt = 1;
        auio.uio_resid = size;
        auio.uio_rw = UIO_WRITE;
        auio.uio_segflg = UIO_SYSSPACE;


	if(!ignore_timeo)
        {
                while(count++ < COUNT_MAX)
                {
                        if(so->so_snd.sb_cc > so->so_snd.sb_hiwat ||
                           so->so_snd.sb_mbcnt > so->so_snd.sb_mbmax  ||
                           sbspace(&so->so_snd) < size)
                        {
#ifdef  NRDDEBUGS
                                printf("nrd_send: Calling event_wait cc %d size %d\n",
                                          so->so_snd.sb_cc, size);
#endif /* NRDDEBUGS */
                                event_wait(&e, 1, TIMEOUT);
#ifdef  NRDDEBUGS
                                printf("nrd_send: After event_wait cc %d size %d\n",
                                          so->so_snd.sb_cc, size);
#endif /* NRDDEBUGS */
                        }

                        else    break;
                }
                if(count >= COUNT_MAX)
                {
                        printf("nrd_send:Warning connection timeout\n");
                        return (-1);
                }
        }

        error = sosend(so, (struct mbuf *)0, &auio, (struct mbuf *)0, (struct mbuf *)0, 0);

        if (error) {
		uprintf("WARNING: sosend returned 0x%x\n", error);
		uprintf("Servers not available or network failure ...\n");
	}


#ifdef  NRDDEBUGS
                uprintf("Exiting nrd_send ... \n");
#endif  /*  NRDDEBUGS  */

        return (error);
}


