/*
**
**      File : nrd_strategy.c
**      (C)1996-97  Mike Flouris.
**
**      Description : strategy device driver interface
**         for the nrd (Network RAM-Disk)
**
*/

#include <mach/vm_prot.h>

int nrdstrategy(bp)
struct buf *bp;
{

	int i = 0, bcnt = 0, j = 0;
	int server_no = 0;
	Header  req;
	struct  buf     *ptr;
	char ack = 0;

	/*
	** Check if the device is open 
	*/
#ifdef NRDSPEED
	after = cycle_count();
        uprintf("Entering strategy : %d\n", after - before );

	before = cycle_count();
#endif

#ifdef NRD_INSTRUMENT
	microtime(&nrd_tv1);
#endif

#ifdef NRDDEBUG
	uprintf("NRD STRATEGY major(%d), minor(%d) ...\n", major(bp->b_dev), minor(bp->b_dev));
#endif

	if (! (open_nrd > 0) ) 
		{
		uprintf("Error : Device not open ...\n");
		bp->b_flags |= B_ERROR;
		bp->b_error = ENXIO;
		return (ENXIO);
	}

	/*
	** Checking for incorrect block numbers ...
	*/
	if ( (bp->b_blkno < 0) || (bp->b_blkno > NRD_SIZE_BLOCKS) ) {
		if( bp->b_flags & B_READ )
			bp->b_resid = bp->b_bcount;
		else {
			bp->b_resid = bp->b_bcount;
			bp->b_flags |= B_ERROR;
			bp->b_error = ENOSPC;
		}
		iodone(bp);
		return (ENOSPC);
	}

	/*
	* Check if not a multiple of the disk block size
	*/
	if ( (bp->b_bcount % DEV_BSIZE ) !=0 )
		uprintf("bp->b_bcount NOT modulo ...\n");



	/*
	** Satisfy the read request ...
	*/
	if (bp->b_flags & B_READ) {

		if ( nrd_queue ) {	/* Use queue */

			ptr = nrd_head;

			bp->b_actf = ptr;
	                bp->b_actl = ptr->b_actl;
	                bp->b_actl->b_actf = bp;
	                ptr->b_actl = bp;

			if(proxy_is_sleeping == TRUE)
				wakeup(nrd_head);

		} else {                /* Do not use queue */

		/* Which server has the data ? */
		server_no = ( bp->b_blkno / NRD_HOST_BLOCKS ) ;

		req.type =     (NRD_REQ_BLOCK_READ);	     /* Read request ... */
		req.blocknum = ( bp->b_bcount / DEV_BSIZE ); /* How many blocks ??? */
		req.stblk = ( bp->b_blkno % NRD_HOST_BLOCKS );/* Starting Block ??? */

#ifdef TRACE_DUMP
		/* Debugging ... */
		uprintf("read  %d  %d\n", req.blocknum, req.stblk);
#endif

		/* Send the request to the server */
		if ( nrd_request( (Header *) &req, server_no ) ) 
			uprintf("Error sending request to server ...\n");

		/* Do the really hard stuff ... */
		nrd_rcv(nrdhosts[ server_no ].sockptr, bp->b_un.b_addr, bp->b_bcount, 0);

		/* The read was successful ... */
                bp->b_resid = 0;
		bp->b_flags |= B_DONE;

		iodone( bp );
		}

	} /* Read request */
	
	/*
	** Satisfy the write request ...
	*/
	else {

		if ( nrd_queue ) {	/* Use queue */

			ptr = nrd_head;

			bp->b_actf = ptr;
       		        bp->b_actl = ptr->b_actl;
       		        bp->b_actl->b_actf = bp;
       		        ptr->b_actl = bp;

			if(proxy_is_sleeping == TRUE)
				wakeup(nrd_head);

		} else {                /* Do not use queue */

		/* Which server has the data ? */
		server_no = ( bp->b_blkno / NRD_HOST_BLOCKS ) ;

		req.type =     ( NRD_REQ_BLOCK_WRITE);	       /* Write request ... */
		req.blocknum = ( bp->b_bcount / DEV_BSIZE );   /* How many blocks ??? */
		req.stblk = ( bp->b_blkno % NRD_HOST_BLOCKS ); /* Starting Block ??? */

#ifdef TRACE_DUMP
		/* Debugging ... */
		uprintf("write  %d  %d\n", req.blocknum, req.stblk);
#endif

		/* Send the request to the server */
		if ( nrd_request( (Header *) &req, server_no ) ) 
			uprintf("Error sending request to server ...\n");

		/* Do the really hard stuff ... */
		nrd_send(nrdhosts[ server_no ].sockptr, bp->b_un.b_addr, bp->b_bcount, 0);

#ifdef SYNC_WRITES
					/* Receive the acknowledgement ... */
		nrd_rcv(nrdhosts[ server_no ].sockptr, (char *)&ack, sizeof(char), 0);

					/* Check the acknowledgement ... */
		while ( ack == 0 ) {
					/* Resend the data ... */
		nrd_send(nrdhosts[ server_no ].sockptr, bp->b_un.b_addr, bp->b_bcount, 0);
					/* Receive the acknowledgement ... */
		nrd_rcv(nrdhosts[ server_no ].sockptr, (char *)&ack, sizeof(char), 0);
		}
#endif		/* SYNC_WRITES */

#ifdef  WRITE_TO_LOCAL_DISK
		bp->b_dev = makedev(DISK_MAJOR, DISK_MINOR);
		if ( cdisk_strategy(bp) != ESUCCESS) {
			uprintf("Cdisk_strategy write failed ...\n");
			bp->b_flags |= B_ERROR;
			bp->b_error = ENOSPC;
			return (ENOSPC);
		}
#endif

		/* The write was successful ...*/
		bp->b_resid = 0;
		bp->b_flags |= B_DONE;

		iodone( bp );
		}

	} /* Write request */

	/*
	** Finish the I/O job and return to the syscall ...
	*/

#ifdef NRDSPEED
	after = cycle_count();

	uprintf("Exiting strategy : %d\n", after - before );
        before = cycle_count();
#endif

#ifdef NRDDEBUG
	uprintf("Returning from NRD STRATEGY ...\n");
#endif

	return (ESUCCESS);
}

	/*
	** End of Strategy ...... over .....
	*/

