/*
**
**      File : nrd_write.c
**      (C)1996-97  Mike Flouris.
**
**      Description : write device driver interface
**         for the nrd (Network RAM-Disk)
**
*/

int nrdwrite(dev, uio, flag)
dev_t dev;
struct uio * uio;
int flag;
{

	int i = 0;
	int server_no = 0;
	Header req;
	char * mybuf;
	char ack = 0;

#ifdef NRDSPEED
        after = cycle_count();
        uprintf("Entering write : %d\n", after - before );

        before = cycle_count();
#endif

#ifdef NRDDEBUG
	uprintf("NRD WRITE major(%d), minor(%d) ...\n", major(dev), minor(dev));
#endif

        /*
        ** Check if the device is open
        */
	if (! (open_nrd > 0) ) 
        	{
                uprintf("Error : Device not open ...\n");
                return (ENXIO);
        }

	/*
	** Satisfy the write request ...
	*/
	if ( uio->uio_rw != UIO_WRITE )
		uprintf("*** uio->uio_rw != UIO_WRITE \n");

	if ( (uio->uio_offset >= 0) && (uio->uio_resid >= 0) &&
	   (uio->uio_offset + uio->uio_resid) <= NRD_SIZE_BYTES) {


		/* Which server has the data ? */
#ifdef SEQUENTIAL_BLOCK_DISTRIBUTION
                server_no = ( uio->uio_offset / NRD_HOST_BYTES ) ;
#elif STRIPED_BLOCK_DISTRIBUTION
                server_no = find_server( ((uio->uio_offset / DEV_BSIZE) + 1) );
#endif

                req.type =     (NRD_REQ_BYTE_WRITE);                 /* Write request ... */
                req.blocknum = (uio->uio_resid);                     /* How many Bytes ??? */
                req.stblk =    ( uio->uio_offset % NRD_HOST_BYTES ); /* Starting Byte ??? */


		/* Send the request to the server */
		if ( nrd_request( (Header *) &req, server_no ) )
			uprintf("Error sending request to server ...\n");


                /* Do the really hard stuff ... */
		mybuf = (char *) kalloc( uio->uio_resid );
		if ( copyin( uio->uio_iov->iov_base, mybuf, uio->uio_resid) != 0)
			uprintf("Copyin error ...");

		nrd_send(nrdhosts[ server_no ].sockptr, mybuf, uio->uio_resid, 0);

#ifdef SYNC_WRITES
				/* Receive the acknowledgement ... */
		nrd_rcv(nrdhosts[ server_no ].sockptr, (char *)&ack, sizeof(char) , 0);

				/* Check the acknowledgement ... */
		 while ( ack == 0 ) {
				/* Resend the data ... */
		nrd_send(nrdhosts[ server_no ].sockptr, mybuf, uio->uio_resid, 0);
				/* Receive the acknowledgement ... */
		nrd_rcv(nrdhosts[ server_no ].sockptr, (char *)&ack, sizeof(char), 0 );
		}
#endif          /* SYNC_WRITES */

		kfree(mybuf, uio->uio_resid);

#ifdef  WRITE_TO_LOCAL_DISK
		if( cdisk_write(makedev(DISK_MAJOR, DISK_MINOR), &uio) != ESUCCESS) {
			uprintf("cdisk_write failed ...\n");
			return (EIO);
		}
#endif

		/* Transfer okay ... */
		uio->uio_resid = 0;

	} else 
		uprintf("NRD WRITE : Invalid address (%d + %d) ...\n", uio->uio_offset, uio->uio_resid);


	/*
	** Finish the I/O job and return to the syscall ...
	*/
#ifdef NRDSPEED
        after = cycle_count();

        uprintf("Exiting write : %d\n", after - before );
        before = cycle_count();
#endif

#ifdef NRDDEBUG
	uprintf("Returning from NRD WRITE ...\n" );
#endif

	return (ESUCCESS);
}

	/*
	** End of Write ...... over .....
	*/

