
/*####################################################################
  
  File:                         nrd_server.c
  Author:                       Mike Flouris

  Description:		
    User level server process handling remote block requests.
    It accepts data for block out and sends data for block in.
  Requires :
    The header file of Network Ramdisk device driver ... (nrd.h) 

  ####################################################################
*/

#include <sys/socket.h>			/* Socket definition */
#include <netinet/in.h>			/* Internet socket definition */
#include <netinet/tcp.h>		/* TCP_NODELAY definition */
#include <string.h>			/* Bzero definition */
#include <machine/endian.h>		/* Htonl definition */
#include <fcntl.h>
#include <sys/types.h>
#include <unistd.h>			/* Read/write definitions */
#include <errno.h>			/* Perror definition */
#include <stdio.h>			/* I/O definitions */
#include <sys/param.h>			/* DEV_BSIZE definition */
#include <signal.h>			/* Signal definitions */
#include <sys/time.h>
#include <sys/ioctl_compat.h>		/* DEVGEOMST ... */
#include <mach.h>			/* Definitions for vm_statistics */

#include "../nrd/nrd.h"		/* Network RamDisk include file ... */

/* Produce trace dumps on DISK */
#define TRACE_DUMP

#ifdef TRACE_DUMP
	FILE	*tdfile;
	char    *tdfilename;
#endif

/*#define SERVER_PORT		2121*/
#define SERVER_PORT		1115

#define NRD_SOCKBUFSIZE  256*1024 	/* TCP buffer size ... */
				
char	nrddisk [NRD_HOST_BYTES];	/* Data repository. */

int	diskfile; 	/* Not depending on WRITE_TO_SERVER_DISK */
char	*diskfilename;	/* so that we can restore from file ... */

void	handle_request();
void    nrd_sighandler();
void	shutdown_server();

int	blocks_sent = 0;
int	bytes_sent  = 0;
int	blocks_received = 0;
int	bytes_received  = 0;



/*
===========================================================================
  Function:		main
  Arguments:	None
  Description:	
	 Creates a socket, binds it to SERVER_PORT and calls
	handle_request for each connection. Every connection should be
	handled by a new process. Let just one connection for the moment.
===========================================================================
*/

int	main(void)
{
	int			sockfd;
	int			newsockfd;
	struct	sockaddr_in	serveraddr;
	struct	sockaddr_in	clientaddr;
	int			clientaddrlen = sizeof(clientaddr);
	int			j = 1, i = 0;
	char	* 		buffer;
	
	signal(SIGINT, nrd_sighandler); /* Perform necessary cleanups */

	/*
	** Creating the socket ... */
	if((sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
		perror("socket_create: ");
		exit(1);
	}

	/*
	** Setting the socket options ... */
	if(setsockopt(sockfd, IPPROTO_TCP, TCP_NODELAY, &j, sizeof(int)) == -1) {
		perror("Seting TCP_NODELAY failed\n");
		exit(1);
	}

	bzero((char *) &serveraddr, sizeof(struct sockaddr_in));
	serveraddr.sin_family 		= 	AF_INET;
	serveraddr.sin_addr.s_addr	=	htonl(INADDR_ANY);
	serveraddr.sin_port		=	htons(SERVER_PORT);
	
	/*
	** Binding the socket ... */
	if(bind(sockfd, (struct sockaddr *) &serveraddr, 
			sizeof(struct sockaddr_in)) < 0 ) {
		extern	int	errno;
		
		printf("Errno is %d\n", errno);
		perror("socket_bind:");
		close(sockfd);
		exit(1);
	}

	/* Initializing the disk file for reliability ... */
	diskfilename = (char *) malloc ( 36 * sizeof(char) );
	if ( gethostname( diskfilename, 30 ) == -1) {
		perror("gethostname :");
		exit(1);
	}
	diskfilename = strcat( diskfilename, ".nrd");

#ifdef WRITE_TO_SERVER_DISK
	diskfile = open( diskfilename, (O_CREAT | O_RDWR | O_NDELAY), S_IRWXU);

	fcntl( diskfile, (FASYNC | FNDELAY) , 0);

	buffer = (char *) malloc(  DEV_BSIZE );
	for ( i = 0; i < NRD_HOST_BLOCKS; i++)
		if ( write( diskfile, buffer, DEV_BSIZE) == -1) {
			perror("Initialising Diskfile:");
			return;
		}
	free( buffer );
#endif

	/*
	** Listening to the socket ... */
	listen(sockfd, 5);
	
	while(1) {
		if((newsockfd = accept(sockfd, (struct sockaddr *) &clientaddr,
			&clientaddrlen)) < 0) {
			perror("socket_accept:");
			close(sockfd);
			exit(1);
		} else { 
		if(setsockopt(newsockfd, IPPROTO_TCP, TCP_NODELAY, &j,sizeof(int))==-1)
                {
                        perror("Seting TCP_NODELAY failed ");
                        exit(1);
                }

#ifdef  SET_TCP_BUFFER			/* Hangs machine if no space available ... */
		j = NRD_SOCKBUFSIZE;
                if(setsockopt(newsockfd, SOL_SOCKET, SO_SNDBUF, &j, sizeof(int)) == -1)
                {
                        perror("Seting SND_BUF failed ");
                        exit(1);
                }
                j = NRD_SOCKBUFSIZE;
                if(setsockopt(newsockfd, SOL_SOCKET, SO_RCVBUF, &j, sizeof(int)) == -1)
                {
                        perror("Seting RCV_BUF failed ");
                        exit(1);
                }
#endif
			printf("Got a Call from 0x%x ...\n", clientaddr.sin_addr.s_addr);
			handle_request(newsockfd);
		}
	}

	return 0;
	
}



/*
===========================================================================
  Function:	nrd_sighandler
  Arguments:	Signal number
  Description:	Responds to signals sent to server.
===========================================================================
*/
void	nrd_sighandler(int signo)
{
	printf("Blocks sent %d(%.2fMB), Blocks received %d(%.2fMB)\n",
		blocks_sent, ((double)(blocks_sent * DEV_BSIZE)) / (double)(1024 * 1024),
		blocks_received, ((double)(blocks_received * DEV_BSIZE)) / (double)(1024 * 1024));
	printf("Bytes  sent %d(%.2fMB), Bytes  received %d(%.2fMB)\n",
		bytes_sent, bytes_sent / (1024 * 1024),
		bytes_received, bytes_received / (1024 * 1024));
	printf("Total data transferred : %.2fMB\n", ((double)(blocks_sent + blocks_received) * DEV_BSIZE + bytes_sent + bytes_received ) / (double)(1024 * 1024) );

#ifdef WRITE_TO_SERVER_DISK
	if ( close( diskfile ) == -1 )
		perror("Diskfile");
#endif
#ifdef 	TRACE_DUMP
	fclose( tdfile );
#endif
	exit( 0 );
}


/*
====================================================================
  Function:	handle_request
  Arguments:	Socket file descriptor
  Description:	Handles a connection performing blockin/blockout 
		requests.
====================================================================
*/
void	handle_request(int fd)
{
	unsigned 	request;	/* Type of request (read/write) */
	unsigned 	num_blocks, num_bytes;	/* Number of requested blocks/bytes */
	unsigned	blkno, byteno;	/* Number of starting block / byte */
	Header		transfer;
	FILE	       *errfp;		/* Error log file */
	char	       *errfpname;	/* Filename of error log file */
	int		i, j;
	struct vm_statistics    srv_vmstat;     /* Results from vm_statistics */
	char		ack = 0;		/* Ack flag ... */
	
#ifdef	NRD_INSTRUMENT
	struct	timeval tv1, tv2;
	struct	timezone tz;
#endif	/*  NRD_INSTRUMENT  */	


	/* Seting TCP_NODELAY ... */
	j  = 1;
	if(setsockopt(fd, IPPROTO_TCP, TCP_NODELAY, &j, sizeof(int)) == -1) {
		perror("Seting TCP_NODELAY failed\n");
		exit(1);
	}
	
#ifdef TRACE_DUMP
	/* Initialize Trace Dump file ... */
	tdfilename = (char *) malloc ( 32 * sizeof(char) );
	if ( gethostname( tdfilename, 26 ) == -1) {
		perror("gethostname :");
		exit(1);
	}
	strcat( tdfilename, ".td");
	tdfile = fopen( tdfilename, "w");
#endif


	while (1) { 	/* One handle_request per client ... */

#ifdef	NRD_INSTRUMENT
		gettimeofday(&tv1, &tz);
#endif  /*  NRD_INSTRUMENT  */

	i = 0;
	while( i < 3 * sizeof(int) ) {
		j = read(fd, (char *)(&transfer)+ i, 3 * sizeof(int) - i);
		if (j == -1) {
			fprintf(stderr, "Read request: Failed...\n");
			close(fd);
			exit(1);
		}
		i += j;
	}

	request = ( transfer.type );
	num_blocks = ( transfer.blocknum );
	num_bytes  = ( transfer.blocknum );
	blkno	= ( transfer.stblk );
	byteno	= ( transfer.stblk );


	/* Sending Acknowledgment */
#ifdef SERVER_ACK
			/* Sending acknowledgement ... */
	i = 0;
	while( i < 3 * sizeof(int) ) {
		j = write(fd, (char *)(&transfer)+ i, 3 * sizeof(int) - i);
		if (j == -1) {
			fprintf(stderr, "Ack request: Failed...\n");
			close(fd);
			exit(1);
		}
		i += j;
	}
	if ( (j = read(fd, (char *)(&ack), sizeof(char)) ) == -1) {
		fprintf(stderr, "Ack request: Failed...\n");
		close(fd);
		exit(1);
	}
		/* Check answer ... */
	if ( ack != 1 ) {
		printf("Wrong request - resending ...\n");
		continue;
	}
#endif


#ifdef	NRD_INSTRUMENT
	gettimeofday(&tv2, &tz);
	printf("Time to read request in usec %d\n", 
		   tv2.tv_usec - tv1.tv_usec + (tv2.tv_sec - tv1.tv_sec)*1000000);
#endif  /*  NRD_INSTRUMENT  */

				/* Satisfy the request ... */
	switch(request)
	{

/* ************************************ */
	case NRD_REQ_BLOCK_WRITE:
/* ************************************ */
		blocks_received += num_blocks;
#ifdef	NRDDEBUGS
		printf("NRD_REQ_BLOCK_WRITE request ...\n");
		printf("Blockout blkno %d, num_blocks %d\n", blkno, num_blocks);
#endif  /*  NRDDEBUGS  */

#ifdef TRACE_DUMP
		fprintf( tdfile, "write  %d  %d\n", num_blocks, blkno );
#endif

		if ( (blkno + num_blocks) > NRD_HOST_BLOCKS) {
			printf("******* OUT OF BOUNDS (req:%d, block:%d)...\n",
				request, (blkno + num_blocks) );
		break;
                }

		i = 0;
		while(i < num_blocks * DEV_BSIZE) {	/* Read the data */
			if((j =read(fd, (char *)&(nrddisk[ blkno * DEV_BSIZE ]) + i, 
						num_blocks * DEV_BSIZE - i)) == -1) {
				fprintf(stderr, "Read data failed\n");
				close(fd);
				return;
			}
			i += j;
		}

#ifdef	NRDDEBUGS
		printf("Transfer ended ...\n");
#endif	/*  NRDDEBUGS  */			

#ifdef WRITE_TO_SERVER_DISK
		if ( lseek( diskfile, (blkno * DEV_BSIZE) , SEEK_SET) == -1 ) {
			perror("Lseek to file:");
			/* close ( diskfile );
			return; */
		}
		if ( write( diskfile, (nrddisk[ blkno * DEV_BSIZE ]),
				 (num_blocks * DEV_BSIZE) ) == -1) {
			perror("Writing to disk:");
			/*close ( diskfile );
			return; */
		}
#endif

#ifdef SYNC_WRITES
		ack = 1;
		if( write(fd, (char *)&ack, sizeof(char) )  == -1) {
			fprintf(stderr, "Write ACK failed ...\n");
			close(fd);
			return;
		}
#endif 		/* SYNC_WRITES ... */

#ifdef	NRD_INSTRUMENT
		gettimeofday(&tv1, &tz);
		printf("Time to send out %d blocks in usec %d\n",num_blocks, 
			tv1.tv_usec - tv2.tv_usec + (tv1.tv_sec - tv2.tv_sec)*1000000);
#endif	/*  NRD_INSTRUMENT  */
		break;
			

/* ************************************ */
	case NRD_REQ_BLOCK_READ:
/* ************************************ */
		blocks_sent += num_blocks;
#ifdef	NRDDEBUGS
		printf("NRD_REQ_BLOCK_READ request ...\n");
		printf("Blockin blkno %d, num_blocks %d\n", blkno, num_blocks);

		printf("Starting transfer ...\n");
#endif  /*  NRDDEBUGS  */

#ifdef TRACE_DUMP
		fprintf( tdfile, "read  %d  %d\n", num_blocks, blkno );
#endif

		/* Break on Sync () */
		if ( (blkno == 0) && (num_blocks == 0) ) {
			printf("Syncing ...\n");
			break;
		}

		if ( (blkno + num_blocks) > NRD_HOST_BLOCKS) {
			printf("******* OUT OF BOUNDS (req:%d, block:%d)...\n",
				request, (blkno + num_blocks) );
		break;
                }


		i = 0;
		while( i < num_blocks * DEV_BSIZE ) {
			if((j = write(fd, (char *) &(nrddisk[ blkno * DEV_BSIZE ]) + i, 
					  num_blocks * DEV_BSIZE - i)) == -1) {
				fprintf(stderr, "write data failed\n");
				close(fd);
				return;
			}
			i += j;
		}	
				
#ifdef	NRDDEBUGS
		printf("Transfer ended ...\n");
#endif	/*  NRDDEBUGS  */			

#ifdef	NRD_INSTRUMENT
		gettimeofday(&tv1, &tz);
		printf("Time to send in %d blocks in usec %d\n",num_blocks, 
		   tv1.tv_usec - tv2.tv_usec + (tv1.tv_sec - tv2.tv_sec)*1000000);
#endif	/*  NRD_INSTRUMENT  */
		break;


/* ************************************ */
	case NRD_REQ_BYTE_READ:
/* ************************************ */
		bytes_sent += num_bytes;
#ifdef	NRDDEBUGS
		printf("NRD_REQ_BYTE_READ request ...\n");
		printf("Bytein byteno %d, num_bytes %d\n", byteno, num_bytes);
			 
		printf("Starting transfer ...\n");
#endif  /*  NRDDEBUGS  */

		if ( (byteno + num_bytes) > NRD_HOST_BYTES) {
			printf("******* OUT OF BOUNDS (req:%d, block:%d)...\n",
				request, (byteno + num_bytes) );
		break;
                }


		i = 0;
		while( i < num_bytes ) {
			if((j = write(fd, (char *) &(nrddisk[ byteno ]) + i, 
					  num_bytes - i) ) == -1) {
				fprintf(stderr, "write data failed\n");
				close(fd);
				return;
			}
			i += j;
		}	
				
#ifdef	NRDDEBUGS
		printf("Transfer ended ...\n");
#endif	/*  NRDDEBUGS  */			

#ifdef	NRD_INSTRUMENT
		gettimeofday(&tv1, &tz);
		printf("Time to send in %d bytes in usec %d\n",num_bytes, 
		   tv1.tv_usec - tv2.tv_usec + (tv1.tv_sec - tv2.tv_sec)*1000000);
#endif/*  NRD_INSTRUMENT  */
		break;


/* ************************************ */
	case NRD_REQ_BYTE_WRITE:
/* ************************************ */
		bytes_received += num_bytes;
#ifdef	NRDDEBUGS
		printf("NRD_REQ_BYTE_WRITE request ...\n");
		printf("Byteout byteno %d, num_bytes %d\n", byteno, num_bytes);
			 
		printf("Starting transfer ...\n");
#endif  /*  NRDDEBUGS  */

		if ( (byteno + num_bytes) > NRD_HOST_BYTES) {
			printf("******* OUT OF BOUNDS (req:%d, block:%d)...\n",
				request, (byteno + num_bytes) );
		break;
                }


		i = 0;
		while( i < num_bytes ) {	/* Read the data */
			if((j =read(fd, (char *) &(nrddisk[ byteno ]) + i, 
						num_bytes - i)) == -1) {
				fprintf(stderr, "read data failed\n");
				close(fd);
				return;
			}
			i += j;
		}

#ifdef	NRDDEBUGS
		printf("Transfer ended ...\n");
#endif	/*  NRDDEBUGS  */			

#ifdef WRITE_TO_SERVER_DISK
		if ( lseek( diskfile, (byteno) , SEEK_SET) == -1 ) {
			perror("Lseek to file:");
			/* close ( diskfile );
			return; */
		}
		if ( write( diskfile, (nrddisk[ byteno ]),
				 (num_bytes) ) == -1) {
			perror("Writing to disk:");
			/*close ( diskfile );
			return; */
		}
#endif

#ifdef SYNC_WRITES
		ack = 1;
		if( write(fd, (char *)&ack, sizeof(char) )  == -1) {
			fprintf(stderr, "Write ACK failed ...\n");
			close(fd);
			return;
		}
#endif 		/* SYNC_WRITES ... */

#ifdef	NRD_INSTRUMENT
		gettimeofday(&tv1, &tz);
		printf("Time to send out %d bytes in usec %d\n",num_bytes, 
			tv1.tv_usec - tv2.tv_usec + (tv1.tv_sec - tv2.tv_sec)*1000000);
#endif	/*  NRD_INSTRUMENT  */
	break;
			
/* ************************************ */
	case NRD_REQ_RESTORE:
/* ************************************ */
	printf("Restoring NRD from disk file\n");

	diskfile = open( diskfilename, (O_RDWR | O_NDELAY), S_IRWXU);
	if (diskfile == -1) {
		perror("Opening diskfile:");
		break;
	}
	for ( i = 0; i < NRD_HOST_BLOCKS; i++)
		if ( read( diskfile, (char *) &(nrddisk[ i * DEV_BSIZE ]),
				 DEV_BSIZE) == -1) {
			perror("Restoring Diskfile:");
			break;
		}
	close ( diskfile );
	break;


/* ************************************ */
	case NRD_REQ_SAVE:
/* ************************************ */
	printf("Saving NRD to disk file\n");

	diskfile = open( diskfilename, (O_RDWR | O_NDELAY), S_IRWXU);
	if (diskfile == -1) {
		perror("Opening diskfile:");
		break;
	}
	for ( i = 0; i < NRD_HOST_BLOCKS; i++)
		if ( write( diskfile, (char *) &(nrddisk[ i * DEV_BSIZE ]),
				 DEV_BSIZE) == -1) {
			perror("Saving Diskfile:");
			break;
		}
	close ( diskfile );
	break;

                                /* Asking server on Virtual Memory Statistics */
/* ************************************ */
        case NRD_REQ_STATS:
/* ************************************ */
		if((i = vm_statistics(task_self(), &srv_vmstat)) != KERN_SUCCESS) {
			perror("vm_statistics failed:");
			exit(1);
		}
		if( write(fd, (char *)&srv_vmstat, 13*sizeof(long) )  == -1) {
			fprintf(stderr, "Sending VM stats failed ...\n");
			close(fd);
			return;
		}
	break;

                                /* Shutting down this server */
/* ************************************ */
        case NRD_REQ_STOP_SERVER:
/* ************************************ */
		printf("Shutting down ...\n");
		return;
	break;


/* ************************************ */
	default:
/* ************************************ */
		fprintf(errfp, "Invalid request %d|%d|%d\n", 
			(transfer.type),(transfer.blocknum),(transfer.stblk) );
	break;
	}

#ifdef	NRDDEBUGS
	printf("Restarting loop ... \n");
#endif	/*  NRDDEBUGS  */			
	}  /* while (1) ... */
	
	fclose( errfp );
	return ; /* pointless ;) ... */

} /* handle request ... */



