/*
**
**      File : nrd.h 
**      (C)1996  Mike Flouris.
**
**      Description : Header file for the device driver interfaces 
**         of the nrd (Network RamDisk).
**     Some thing MUST be configured here ...
**
**     ATTENTION : This is for the Linux version !
*/


/*
** ##################################
**      Debugging info level 1
**    Mostly debugging garbage ...
** ################################## */
/* #define NRDDEBUG YES */
#undef NRDDEBUG   /* Configurable ... */



/*
** ###############################################
**     NRD Reliability / Speed Features ...
**  Uncomment ONE ONLY of the following defines
**        IF other than the default ...
** ################################################ */
/*
 ** Level 1:  NRD_UNRELIABLE_NOCACHE
         Sends writes immediately.
         Doesn't use a write cache ...
 ** Level 2:  NRD_UNRELIABLE_CACHE
         Sends batched (cached) writes.
         Uses a write cache ... (MAX SPEED - DEFAULT)
 ** Level 3:  NRD_RELIABLE_PARITY
         Uses parity caching reliability.
         No failure if one server crashes ...
         Using a write (parity) cache to gain
         performance from less network transfers ...
 **/

#undef  NRD_UNRELIABLE_NOCACHE

/* Default level  :  2 (max speed) */
#define NRD_UNRELIABLE_CACHE

#undef  NRD_RELIABLE_PARITY

/*
** ####################################
**   How big is the cache ? ( 8 KB )
** #################################### */
#if defined(NRD_UNRELIABLE_CACHE) || defined(NRD_RELIABLE_PARITY)
#define CACHE_SIZE   8 /* KBytes. Configurable */
#define CACHE_SETS   4 /* Number of adaptive parity cache sets.
                          Configurable (better leave it untouched) */
#define CACHE_TIMEOUT   2 /* Timeout of adaptive parity caching.
                             The value is the times the Victim set will
                             be sent ... Configurable */
#endif

/*
** ################################
**    How many server hosts ???
** ################################ */
#define NRDHOSTS 1    /* Configurable from the command line option.
                       * This is the default value ... */

/*
** ######################################
**   Size of NRD per server : 20 MBytes
** ###################################### */
#define NRD_HOST_MBYTES  20 /* Configurable */

/*
** ##############################################
**     NRD default hostname (Configurable) ...
** ############################################## */
#define NRD_DEF_HOSTNAME    "titan.ics.forth.gr"

/*
** ############################################
**      The port number of NRD Servers ...
**    (Configurable but must recompile both
**           client and server ... )
** ############################################ */
#define NRD_SERVER_PORT           2121        /* Port number 2121  = 0x0849 */

/*
** ########################################
**  The client's disk attributes ...
** ######################################## */
#define MAJOR_NR        121   /* Configurable, but must match /dev/nrd entry */
#define DEVICE_NAME     "nrd"

/*
** ########################################
**         The cleaning threshold 
**   (max cache buffers to clean at once)
** ######################################## */
#define NRD_MAX_CLEANING    30



/* ******************************************************************
   ##################################################################
         Configurable part ends here. DO NOT edit below ...
   ##################################################################
   ******************************************************************/



/*
** ########################################
**   Programming & Debugging Defines ...
**       Don't need editing :) 
** ########################################
*/
#if !defined(YES)
#define YES 1
#define NO  0
#endif

#if !defined(ESUCCESS)
#define ESUCCESS 0
#endif

#if !defined(TRUE)
#define FALSE   0
#define TRUE    1
#endif /* !defined(TRUE) */

/*
** ############################################
**   Redefine the number of host bytes in the
**       case of Reliable parity caching.
**          (Using 10% more blocks ...)
** ############################################ */
#ifndef NRD_RELIABLE_PARITY
#define NRD_HOST_BYTES  (NRD_HOST_MBYTES * 1024 * 1024)
#else
#define NRD_HOST_BYTES  ((NRD_HOST_MBYTES*1024*1024)+(SYS_BLOCK_SIZE*(((NRD_HOST_MBYTES*1024*1024)/SYS_BLOCK_SIZE)/10)))
#endif

/*
** ######################################
**    NRD Block Sizes ... (DO NOT EDIT)
** ###################################### */
/* DO NOT TOUCH THIS ... */
#define SYS_BLOCK_SIZE  512     /* The block size of the Linux
                                   system ... DO NOT EDIT ! */
/* DO NOT TOUCH THIS ALSO ... */
#define NRD_BLOCK_SIZE  1024    /* The block size of the NRD device
                                   on Linux ... DO NOT EDIT ! */
/* NRD_BLOCK_SIZE = 2*SYS_BLOCK_SIZE
 * Much code is using this analogy directly.
 * Changing this will result in failure of the code ... */
#define NRD_HOST_BLOCKS ( NRD_HOST_BYTES / SYS_BLOCK_SIZE ) /* DO NOT EDIT ! */

/*
** ################################
**   Do we have a write cache ???
** ################################ */
#if defined(NRD_UNRELIABLE_CACHE) || defined(NRD_RELIABLE_PARITY)
#define CACHE_SIZE_BYTES    (CACHE_SIZE * NRD_BLOCK_SIZE) /* DO NOT EDIT ! */
#define CACHE_SIZE_BLOCKS   (CACHE_SIZE_BYTES / SYS_BLOCK_SIZE) /* DO NOT EDIT ! */
#endif

/*
** ################################
**       TCP/IP defines ...
** ################################ */
#undef  SET_TCP_BUFFER                  /* Maximize TCP buffer ??? (default: no) */
#define NRD_SOCKBUFSIZE     256*1024    /* Max TCP buffer size ... */
#define NRD_TIMEOUT             1024    /* Timeout to connect sockets to servers ... */
#define TIMEOUT                 1       /* Polling interval for send/rcv */
#define COUNT_MAX               60000   /* Connection timeout in ms*/

/*
** ################################
     Type of operation requested
** ################################ */
#define NRD_REQ_BLOCK_WRITE         0x0 /* Block write request */
#define NRD_REQ_BLOCK_READ          0x1 /* Block read  request */
#define NRD_REQ_BYTE_WRITE          0x2 /* Byte  write request */
#define NRD_REQ_BYTE_READ           0x3 /* Byte  read  request */
#define NRD_REQ_CACHE_WRITE         0x4 /* Block Cache Write request */
#define NRD_REQ_RLB_CACHE_WRITE     0x5 /* Reliable Block Cache Write request */
#define NRD_REQ_BLOCK_READV         0x6 /* Block Read request with Virtual Block numbers */
#define NRD_REQ_STATS               0x7 /* Server VM Stats request ... */
#define NRD_REQ_STOP_SERVER         0x8 /* Shutdown server request ... */

/*
** ################################
     Type of block on block_map
        (for RELIABLE_PARITY)
** ################################ */
#define NRD_PBLOCK_EMPTY            0   /* Physical Block empty */
#define NRD_PBLOCK_FULL             1   /* Physical Block full */
#define NRD_PBLOCK_INVALID          2   /* Physical Block invalid (stale data) */

/*
** ############################################
**  Error Number for Lost connection 
** ############################################ */
#define ETIMEO      128 /* Error value: Connection timeout */

/*
** ##################################
     NRD special ioctl commands ... 
** ################################## */
#define NRD_IOCTL_INIT          _IO(0x21, 1)
    /* This queries NRD servers for VM statistics ... */
#define NRD_IOCTL_STATS         _IO(0x21, 2)
    /* This stops the server and the queue ... */
#define NRD_IOCTL_STOP          _IO(0x21, 3)
#define NRD_IOCTL_SERVERS       _IOW(0x21, 4, Nrd_Server_Info)
#define NRD_IOCTL_USER          _IOW(0x21, 5, Nrd_User_Info)
#define NRD_IOCTL_USER_READ     _IOR(0x21, 6, Nrd_User_Info)


/*
** #########################
**       Global Typedefs 
** ######################### */
typedef struct IO_Request {
        /* Type of Request : R/W */
    unsigned type;
        /* Number of Blocks ... */
    unsigned blocknum;
        /* Starting Block ... */
    unsigned stblk;
} IO_Request;

    /* This is used to pass info to user-level thread */
typedef struct Nrd_User_Info {
    IO_Request req;
    char   data[NRD_BLOCK_SIZE];
} Nrd_User_Info;

#ifdef NRD_UNRELIABLE_CACHE
    /* The cache for each server ... */
typedef struct Nrd_Cache {
    unsigned   block_index[ CACHE_SIZE_BLOCKS ];
    char       block_data [ CACHE_SIZE_BYTES ];
} Nrd_Cache;
#elif defined(NRD_RELIABLE_PARITY)
    /* The reliable cache for each server ... */
typedef struct Nrd_Reliable_Cache {
    int         p_block_index[ CACHE_SIZE_BLOCKS ];
    int         v_block_index[ CACHE_SIZE_BLOCKS ];
    char        block_data [ CACHE_SIZE_BYTES ];
    unsigned    timestamp;
} Nrd_Reliable_Cache;

typedef struct Nrd_BRead {
    int         v_block_index[ NRD_BLOCK_SIZE/SYS_BLOCK_SIZE ];
    char        block_data [ NRD_BLOCK_SIZE ];
} Nrd_BRead;
#endif


